@@ -6,6 +6,7 @@ pub enum Platform {
     LinuxAmd64,
     LinuxAarch64,
     MacOsAmd64,
+    MacOsAarch64,
     WindowsAmd64,
     Unsupported,
 }
@@ -16,6 +17,7 @@ impl ToString for Platform {
             Platform::LinuxAmd64 => "linux-amd64".to_string(),
             Platform::LinuxAarch64 => "linux-aarch64".to_string(),
             Platform::MacOsAmd64 => "macosx-amd64".to_string(),
+            Platform::MacOsAarch64 => "macosx-aarch64".to_string(),
             Platform::WindowsAmd64 => "windows-amd64".to_string(),
             Platform::Unsupported => "Unsupported-platform".to_string(),
         }
@@ -27,9 +29,8 @@ pub fn platform() -> Platform {
     match (env::consts::OS, env::consts::ARCH) {
         ("linux", "x86_64") => Platform::LinuxAmd64,
         ("linux", "aarch64") => Platform::LinuxAarch64,
-        // NOTE: Relaxed requirement on target architecture here
-        // to support M1 macs with Rosetta
-        ("macos", _) => Platform::MacOsAmd64,
+        ("macos", "x86_64") => Platform::MacOsAmd64,
+        ("macos", "aarch64") => Platform::MacOsAarch64,
         ("windows", "x86_64") => Platform::WindowsAmd64,
         _ => Platform::Unsupported,
     }
@@ -45,6 +46,12 @@ mod tests {
         assert_eq!(platform(), Platform::LinuxAmd64);
     }
 
+    #[test]
+    #[cfg(all(target_os = "linux", target_arch = "aarch64"))]
+    fn get_platform() {
+        assert_eq!(platform(), Platform::LinuxAarch64);
+    }
+
     #[test]
     #[cfg(all(target_os = "macos", target_arch = "x86_64"))]
     fn get_platform() {
@@ -54,7 +61,7 @@ mod tests {
     #[test]
     #[cfg(all(target_os = "macos", target_arch = "aarch64"))]
     fn get_platform() {
-        assert_eq!(platform(), Platform::MacOsAmd64);
+        assert_eq!(platform(), Platform::MacOsAarch64);
     }
 
     #[test]
