@@ -3,14 +3,15 @@ use reqwest::get;
 use semver::Version;
 use serde::{
     de::{self, Deserializer},
-    Deserialize,
+    Deserialize, Serialize,
 };
-use std::collections::HashMap;
+use std::collections::BTreeMap;
 use url::Url;
 
 use crate::{error::SolcVmError, platform::Platform};
 
 const SOLC_RELEASES_URL: &str = "https://binaries.soliditylang.org";
+
 const OLD_SOLC_RELEASES_DOWNLOAD_PREFIX: &str =
     "https://raw.githubusercontent.com/crytic/solc/master/linux/amd64";
 
@@ -31,6 +32,14 @@ static LINUX_AARCH64_RELEASES: Lazy<Releases> = Lazy::new(|| {
         .expect("could not parse list linux-aarch64.json")
 });
 
+static MACOS_AARCH64_URL_PREFIX: &str =
+    "https://github.com/roynalnaruto/solc-builds/raw/eb96e79ee82860a55c0bae4fd1c2f279f2c2035c/macosx/aarch64";
+
+static MACOS_AARCH64_RELEASES: Lazy<Releases> = Lazy::new(|| {
+    serde_json::from_str(include_str!("../list/macosx-aarch64.json"))
+        .expect("could not parse list macosx-aarch64.json")
+});
+
 /// Defines the struct that the JSON-formatted release list can be deserialized into.
 ///
 /// {
@@ -48,11 +57,10 @@ static LINUX_AARCH64_RELEASES: Lazy<Releases> = Lazy::new(|| {
 /// }
 ///
 /// Both the key and value are deserialized into semver::Version.
-#[derive(Clone, Debug, Default, Deserialize)]
+#[derive(Clone, Debug, PartialEq, Eq, Default, Serialize, Deserialize)]
 pub struct Releases {
     pub builds: Vec<BuildInfo>,
-    #[serde(deserialize_with = "de_releases")]
-    pub releases: HashMap<Version, String>,
+    pub releases: BTreeMap<Version, String>,
 }
 
 impl Releases {
@@ -80,43 +88,34 @@ impl Releases {
 }
 
 /// Build info contains the SHA256 checksum of a solc binary.
-#[derive(Clone, Debug, Deserialize)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
 pub struct BuildInfo {
-    #[serde(deserialize_with = "version_from_string")]
     pub version: Version,
-    #[serde(deserialize_with = "from_hex_string")]
+    #[serde(with = "hex_string")]
     pub sha256: Vec<u8>,
 }
 
-/// Helper to parse hex string to a vector.
-fn from_hex_string<'de, D>(deserializer: D) -> Result<Vec<u8>, D::Error>
-where
-    D: Deserializer<'de>,
-{
-    let str_hex = String::deserialize(deserializer)?;
-    let str_hex = str_hex.trim_start_matches("0x");
-    hex::decode(str_hex).map_err(|err| de::Error::custom(err.to_string()))
-}
+/// Helper serde module to serialize and deserialize bytes as hex.
+mod hex_string {
+    use super::*;
+    use serde::Serializer;
+    pub fn deserialize<'de, D>(deserializer: D) -> Result<Vec<u8>, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let str_hex = String::deserialize(deserializer)?;
+        let str_hex = str_hex.trim_start_matches("0x");
+        hex::decode(str_hex).map_err(|err| de::Error::custom(err.to_string()))
+    }
 
-/// Helper to parse string to semver::Version.
-fn version_from_string<'de, D>(deserializer: D) -> Result<Version, D::Error>
-where
-    D: Deserializer<'de>,
-{
-    let str_version = String::deserialize(deserializer)?;
-    Version::parse(&str_version).map_err(|err| de::Error::custom(err.to_string()))
-}
-
-/// Custom deserializer that deserializes a map of <String, String> to <Version, Version>.
-fn de_releases<'de, D>(deserializer: D) -> Result<HashMap<Version, String>, D::Error>
-where
-    D: Deserializer<'de>,
-{
-    #[derive(PartialEq, Eq, Hash, Deserialize)]
-    struct Wrapper(#[serde(deserialize_with = "version_from_string")] Version);
-
-    let v = HashMap::<Wrapper, String>::deserialize(deserializer)?;
-    Ok(v.into_iter().map(|(Wrapper(k), v)| (k, v)).collect())
+    pub fn serialize<T, S>(value: &T, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+        T: AsRef<[u8]>,
+    {
+        let value = hex::encode(value);
+        serializer.serialize_str(&value)
+    }
 }
 
 /// Blocking version fo [`all_realeases`]
@@ -141,6 +140,10 @@ pub async fn all_releases(platform: Platform) -> Result<Releases, SolcVmError> {
         return Ok(LINUX_AARCH64_RELEASES.clone());
     }
 
+    if platform == Platform::MacOsAarch64 {
+        return Ok(MACOS_AARCH64_RELEASES.clone());
+    }
+
     let releases = get(format!(
         "{}/{}/list.json",
         SOLC_RELEASES_URL,
@@ -202,6 +205,20 @@ pub fn artifact_url(
         ));
     }
 
+    if platform == Platform::MacOsAarch64 {
+        if MACOS_AARCH64_RELEASES.releases.contains_key(version) {
+            return Ok(Url::parse(&format!(
+                "{}/{}",
+                MACOS_AARCH64_URL_PREFIX, artifact
+            ))?);
+        } else {
+            return Err(SolcVmError::UnsupportedVersion(
+                version.to_string(),
+                platform.to_string(),
+            ));
+        }
+    }
+
     Ok(Url::parse(&format!(
         "{}/{}/{}",
         SOLC_RELEASES_URL,
@@ -226,13 +243,37 @@ mod tests {
         assert_eq!(LINUX_AARCH64_RELEASES.builds.len(), 43);
     }
 
+    #[test]
+    fn test_macos_aarch64() {
+        assert_eq!(MACOS_AARCH64_RELEASES.releases.len(), 4);
+        assert_eq!(MACOS_AARCH64_RELEASES.builds.len(), 4);
+    }
+
     #[tokio::test]
-    async fn test_all_releases_macos() {
+    async fn test_all_releases_macos_amd64() {
         assert!(all_releases(Platform::MacOsAmd64).await.is_ok());
     }
 
     #[tokio::test]
-    async fn test_all_releases_linux() {
+    async fn test_all_releases_macos_aarch64() {
+        assert!(all_releases(Platform::MacOsAarch64).await.is_ok());
+    }
+
+    #[tokio::test]
+    async fn test_all_releases_linux_amd64() {
         assert!(all_releases(Platform::LinuxAmd64).await.is_ok());
     }
+
+    #[tokio::test]
+    async fn test_all_releases_linux_aarch64() {
+        assert!(all_releases(Platform::LinuxAarch64).await.is_ok());
+    }
+
+    #[tokio::test]
+    async fn releases_roundtrip() {
+        let releases = all_releases(Platform::LinuxAmd64).await.unwrap();
+        let s = serde_json::to_string(&releases).unwrap();
+        let de_releases: Releases = serde_json::from_str(&s).unwrap();
+        assert_eq!(releases, de_releases);
+    }
 }
